from django_filters.rest_framework import DjangoFilterBackend
from rest_framework.filters import SearchFilter, OrderingFilter
from rest_framework.renderers import BrowsableAPIRenderer
from rest_framework.renderers import JSONRenderer
from rest_framework_csv.renderers import PaginatedCSVRenderer
from rest_framework.viewsets import ReadOnlyModelViewSet
from drf_renderer_xlsx.renderers import XLSXRenderer


from .filters import create_model_filterset_class
from .mixins import FileNameMixin
from .renderers import NonPaginatedCSVRenderer, NonPaginatedXLSXRenderer
from .serializers import create_model_serializer_class


def create_model_viewset_class(
    model,
    serializer=None,
    filterset_class=None,
    queryset=None,
    debug=False,
    renderer_classes=(
        BrowsableAPIRenderer,
        JSONRenderer,
        PaginatedCSVRenderer,
        NonPaginatedCSVRenderer,
        XLSXRenderer,
        NonPaginatedXLSXRenderer,
    ),
):
    if not model:
        raise Exception("You must pass in a model")

    model_name = model.__name__
    if debug:
        print("model_name:", model_name)

    if model and not filterset_class:
        filterset_class = create_model_filterset_class(model=model)

    if model and not serializer:
        serializer = create_model_serializer_class(model=model)

    if not queryset:
        queryset = model.objects.all()

    fields = model._meta.get_fields(include_hidden=True)
    if debug:
        print("fields:", fields)

    text_fields = [
        f.name for f in fields if f.__class__.__name__ in ("CharField", "TextField")
    ]
    if debug:
        print("text_fields:", text_fields)

    viewset_name = model_name + "ViewSet"
    if debug:
        print("viewset_name:", viewset_name)

    defs = {
        "renderer_classes": renderer_classes,
        "queryset": queryset,
        "serializer_class": serializer,
        "filter_backends": [DjangoFilterBackend, SearchFilter, OrderingFilter],
        "filterset_class": filterset_class,
        "search_fields": text_fields,
        "ordering_fields": "__all__",
    }

    viewset = type(viewset_name, (FileNameMixin, ReadOnlyModelViewSet,), defs)
    if debug:
        print("viewset:", viewset)

    return viewset



from rest_framework_csv.renderers import CSVRenderer
from drf_renderer_xlsx.renderers import XLSXRenderer


class NonPaginatedCSVRenderer(CSVRenderer):
    format = "csv (non-paginated)"

    def render(self, data, accepted_media_type, renderer_context):
        response = renderer_context["response"]

        if response.status_code != 200:
            return ""

        view = renderer_context["view"]
        request = renderer_context["request"]

        queryset = view.queryset
        query_params = request.query_params.dict()
        query_params.pop("format", None)

        # ignore the page parameter
        query_params.pop("page", None)

        # filter out keys with empty strings
        filters = {k: v for k, v in query_params.items() if v != ""}

        queryset = queryset.filter(**filters)

        serializer = view.serializer_class

        data = serializer(queryset, many=True).data

        return super(NonPaginatedCSVRenderer, self).render(
            data, accepted_media_type, renderer_context
        )


class NonPaginatedXLSXRenderer(XLSXRenderer):
    format = "xlsx (non-paginated)"

    def render(self, data, accepted_media_type, renderer_context):
        response = renderer_context["response"]

        if response.status_code != 200:
            return ""

        view = renderer_context["view"]
        request = renderer_context["request"]

        queryset = view.queryset
        query_params = request.query_params.dict()
        query_params.pop("format", None)

        # ignore the page parameter
        query_params.pop("page", None)

        # filter out keys with empty strings
        filters = {k: v for k, v in query_params.items() if v != ""}

        queryset = queryset.filter(**filters)

        serializer = view.serializer_class

        data = serializer(queryset, many=True).data

        return super(NonPaginatedXLSXRenderer, self).render(
            data, accepted_media_type, renderer_context
        )



from rest_framework.response import Response


class FileNameMixin(object):
    def get_filename(self, request, response):
        try:
            return request.path_info.replace("/", "_").strip("_")
        except:
            pass

        try:
            return self.queryset.model.__name__
        except:
            return "download"

    def finalize_response(self, request, response, *args, **kwargs):
        response = super(FileNameMixin, self).finalize_response(
            request, response, *args, **kwargs
        )
        if isinstance(response, Response):
            if "xlsx" in response.accepted_renderer.format:
                filename = self.get_filename(request, response) + ".xlsx"
            elif "csv" in response.accepted_renderer.format:
                filename = self.get_filename(request, response) + ".csv"
            else:
                filename = None

            if filename:
                response["content-disposition"] = f"attachment; filename={filename}"

        return response
